###################################################################################################################################################
# Copyright 2015 Orchid Systems 
# -----------------------------
# Permitted for use with Orchid Systems Modules only.
# The code is supplied on an as is basis for training and demonstration purposes only and is not supported by Orchid Systems.
# If this code is deployed in a live production environment it is the responsibility of the End User to ensure 
# that it is operating as required.
###################################################################################################################################################


#################### Import Modules ###########################################
from accpac import *

import time
import datetime
import subprocess
from datetime import datetime

###############################################################################

# Old batch status
orig = "0"
origDesc = ""

rotoDict = {"GL0008":["BATCHSTAT", "4", "BATCHID"], "AP0020":["BTCHSTTS", "3", "CNTBTCH"],
            "AP0030":["BATCHSTAT", "3", "CNTBTCH"], "AR0031":["BTCHSTTS", "3", "CNTBTCH"], 
			"AR0041":["BATCHSTAT", "3", "CNTBTCH"], "AR0140":["BTCHSTTS", "3", "CNTBTCH"]}

def onOpen():
	rotoList = ["GL0008", "AP0020", "AP0030", "AR0031", "AR0041", "AR0140"]

	if (rotoList.count(rotoID) == 0):
		return 0 # don't do anything since the view is not G/L Batch, A/P Batch, or A/R Batch.

	global rotoDict
	#fld = rotoDict[rotoID][1]

	return Continue

#------------------------------------------------------------
# Audit on generate. That is on create.

def onAfterRecordGenerate(result, flags):
	rvspyTrace("onAfterRecordGenerate result=" + str(result) + " flags=" + str(flags))
	if result == 0:
		writeToAudit("GENERATE")


def onBeforeUpdate():
	global orig
	global rotoDict

	#showMessage("onBeforeUpdate")

	#batchstat_fld = rotoDict[rotoID][0]

	if (rotoID == "GL0008"):
		orig = ReplaceFields("{ORIGINAL_BATCHSTAT}")
	elif (rotoID == "AP0020"):
		orig = ReplaceFields("{ORIGINAL_BTCHSTTS}")
	elif (rotoID == "AP0030"):
		orig = ReplaceFields("{ORIGINAL_BATCHSTAT}")
	elif (rotoID == "AR0031"):
		orig = ReplaceFields("{ORIGINAL_BTCHSTTS}")
	elif (rotoID == "AR0041"):
		orig = ReplaceFields("{ORIGINAL_BATCHSTAT}")
	elif (rotoID == "AR0140"):
		orig = ReplaceFields("{ORIGINAL_BTCHSTTS}")

	#showMessage(orig)
	
	return Continue


#------------------------------------------------------------
# Audit on update AND on post.

def onAfterUpdate(result):
	#showMessage("onAfterUpdate result=" + str(result))

	global orig
	global rotoDict

	# Retrieve the name of the batch status field based on the current view.
	fld = rotoDict[rotoID][0]
	postedStatus = rotoDict[rotoID][1]

	if (result == 0):
		if (rotoID == "GL0008"):
			curr = ReplaceFields("{BATCHSTAT}")
		elif (rotoID == "AP0020"):
			curr = ReplaceFields("{BTCHSTTS}")
		elif (rotoID == "AP0030"):
			curr = ReplaceFields("{BATCHSTAT}")
		elif (rotoID == "AR0031"):
			curr = ReplaceFields("{BTCHSTTS}")
		elif (rotoID == "AR0041"):
			curr = ReplaceFields("{BATCHSTAT}")
		elif (rotoID == "AR0140"):
			curr = ReplaceFields("{BTCHSTTS}")

		#showMessage("value: " + curr)
		if orig != curr:
			# check status.
			if curr == postedStatus:
				# Posted
				writeToAudit("POSTED")
		else:
			# some other field changed. Record an update.
			writeToAudit("UPDATED")


def writeToAudit(sAction):

	dt = datetime.now()

	# open view.
	VILOG = openView("VI0002", 1)
	VILOG.recordClear()

	# populate key fields:
	f_datetime = dt.strftime("%Y%m%d%H%M%S%f")[:-2]
	f_viewid = rotoID
	f_change = 0
	f_key = me.get(rotoDict[rotoID][2])
	f_key = int(float(f_key))

	# if rotoid is for G/L:
	appID = rotoID[0:2]
	if (appID == "GL"):
		f_key = "{:06}".format(f_key)
	else:
		f_key = str(f_key)
	
	#showMessage(f_key)

	# check if record exists in VILOG.
	VILOG.put("DATETIME", f_datetime)
	VILOG.put("VIEWID", f_viewid)
	VILOG.put("CHANGE", f_change)
	VILOG.put("KEY", f_key)
	res = VILOG.read()
	while (res == 0):
		# VI audit log record was found. Increment change variable and try again.
		f_change = int(VILOG.get("CHANGE")) + 1

		if f_change > 100:
			return Continue		# Don't write to the audit. Something's gone wrong and we've reached a hundred count. This shouldn't happen so exit and don't write to the VILOG table.

		else:
			VILOG.recordClear()
			VILOG.put("DATETIME", f_datetime)
			VILOG.put("VIEWID", f_viewid)
			VILOG.put("CHANGE", f_change) # has been incremented by 1.
			VILOG.put("KEY", f_key)
			res = VILOG.read()


	# populate other fields:
	f_date = dt.strftime("%Y%m%d")
	f_time = dt.strftime("%H%M%S00")
	f_field = "BATCHSTAT"
	f_userid = user

	if (sAction == "GENERATE"):
		# create record to indicate batch generated.
		f_from = ""
		f_to = "Batch Created."

	elif (sAction == "POSTED"):
		# create record to indicate batch posted.
		f_from = ""
		f_to = "Batch Posted"

	elif (sAction == "UPDATED"):
		# create record to indicate batch updated.
		f_from = ""
		f_to = "Batch Updated"

	VILOG.recordClear()
	VILOG.put("DATETIME", f_datetime)
	VILOG.put("VIEWID", f_viewid)
	VILOG.put("CHANGE", f_change)
	VILOG.put("KEY", f_key)
	VILOG.put("DATE", f_date)
	VILOG.put("TIME", f_time)
	VILOG.put("USERID", f_userid)
	VILOG.put("FIELD", f_field)
	VILOG.put("FROM", f_from)
	VILOG.put("TO", f_to)

	VILOG.insert()

	VILOG.close()

